@@ -1,20 +1,20 @@
-#!/usr/bin/python2.7
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """
 **Project Name:**      MakeHuman
 
-**Product Home Page:** http://www.makehuman.org/
+**Product Home Page:** http://www.makehumancommunity.org/
 
-**Code Home Page:**    https://bitbucket.org/MakeHuman/makehuman/
+**Github Code Home Page:**    https://github.com/makehumancommunity/
 
 **Authors:**           Joel Palmius, Marc Flerackers, Jonas Hauquier
 
-**Copyright(c):**      MakeHuman Team 2001-2017
+**Copyright(c):**      MakeHuman Team 2001-2020
 
 **Licensing:**         AGPL3
 
-    This file is part of MakeHuman (www.makehuman.org).
+    This file is part of MakeHuman Community (www.makehumancommunity.org).
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -46,6 +46,9 @@ from getpath import getSysDataPath, canonicalPath
 import log
 import material
 import animation
+import sys
+from uuid import uuid4
+from mesh_operations import calculateSurface, calculateVolume
 
 from makehuman import getBasemeshVersion, getShortVersion, getVersionStr, getVersion
 
@@ -57,6 +60,10 @@ class Human(guicommon.Object, animation.AnimatedMesh):
 
         self.hasWarpTargets = False
 
+        self._name = ''
+        self._tags = set()
+        self._uuid = ''
+
         self.MIN_AGE = 1.0
         self.MAX_AGE = 90.0
         self.MID_AGE = 25.0
@@ -97,6 +104,48 @@ class Human(guicommon.Object, animation.AnimatedMesh):
         # Make sure that shadow vertices are copied
         self.refreshStaticMeshes()
 
+    def getName(self):
+        return self._name
+
+    def setName(self, name=''):
+        self._name = name
+
+    humanName = property(getName, setName)
+
+    def getTags(self):
+        return self._tags
+
+    def setTags(self, tags):
+        if not isinstance(tags, set):
+            raise ValueError('Parameter must be set type')
+        else:
+            tset = set()
+            for t in tags:
+                tset.add(t[:25]) # Max. tag length is 25
+            self._tags = tset
+
+    tags = property(getTags, setTags)
+
+    def addTag(self, tag=''):
+        self._tags.add(tag.lower()[:25]) # Max. tag length is 25
+
+    def clearTags(self):
+        self._tags.clear()
+
+    def getUuid(self):
+        if self._uuid:
+            return self._uuid
+        else:
+            return ''
+
+    def setUuid(self, _uuid=''):
+        self._uuid = _uuid
+
+    def newUuid(self):
+        self._uuid = str(uuid4())
+        return self._uuid
+
+
     def setProxy(self, proxy):
         oldPxy = self.getProxy()
         oldPxyMesh = self.getProxyMesh()
@@ -230,6 +279,7 @@ class Human(guicommon.Object, animation.AnimatedMesh):
         mesh = self.meshData
         group_mask = np.ones(len(mesh._faceGroups), dtype=bool)
         for g in mesh._faceGroups:
+            g.name = str(g.name) 
             if g.name.startswith('joint-') or g.name.startswith('helper-'):
                 group_mask[g.idx] = False
         face_mask = group_mask[mesh.group]
@@ -325,7 +375,7 @@ class Human(guicommon.Object, animation.AnimatedMesh):
                 proxies.append(pxy)
         if includeHumanProxy and self.proxy:
             proxies.append(self.proxy)
-        for pxy in self._clothesProxies.values():
+        for pxy in list(self._clothesProxies.values()):
             proxies.append(pxy)
         return proxies
 
@@ -575,6 +625,17 @@ class Human(guicommon.Object, animation.AnimatedMesh):
     def getWeight(self):
         return self.weight
 
+    def getBsa(self):
+        return calculateSurface(self.meshData, vertGroups=['body'])/100
+
+    def getVolume(self):
+        return calculateVolume(self.meshData, vertGroups=['body'])/1000
+
+    def getWeightKg(self):
+        # Estimating weight using Mosteller's formula for body surface area estimation
+        bsa = self.getBsa()
+        return bsa * bsa * 3600 / self.getHeightCm()
+
     def _setWeightVals(self):
         self.maxweightVal = max(0.0, self.weight * 2 - 1)
         self.minweightVal = max(0.0, 1 - self.weight * 2)
@@ -874,14 +935,14 @@ class Human(guicommon.Object, animation.AnimatedMesh):
         """
         All modifier objects attached to this human.
         """
-        return self._modifiers.values()
+        return list(self._modifiers.values())
 
     @property
     def modifierNames(self):
         """
         The names of all modifiers available.
         """
-        return self._modifiers.keys()
+        return list(self._modifiers.keys())
 
     def getModifierNames(self):
         """
@@ -902,7 +963,7 @@ class Human(guicommon.Object, animation.AnimatedMesh):
         The names of all groups in which the modifiers of this human are
         classified.
         """
-        return self._modifier_groups.keys()
+        return list(self._modifier_groups.keys())
 
     def getModifiersByGroup(self, groupName):
         """
@@ -1098,7 +1159,7 @@ class Human(guicommon.Object, animation.AnimatedMesh):
 
         # Apply targets to seedmesh coordinates
         itprog = Progress(len(self.targetsDetailStack))
-        for (targetPath, morphFactor) in self.targetsDetailStack.iteritems():
+        for (targetPath, morphFactor) in self.targetsDetailStack.items():
             algos3d.loadTranslationTarget(self.meshData, targetPath, morphFactor, None, 0, 0)
             itprog.step()
 
@@ -1238,6 +1299,10 @@ class Human(guicommon.Object, animation.AnimatedMesh):
 
         self.targetsDetailStack = {}
 
+        self.setUuid('')
+        self.setName('')
+        self.clearTags()
+
         self.setMaterial(self._defaultMaterial)
 
         self.callEvent('onChanging', events3d.HumanEvent(self, 'reset'))
@@ -1391,7 +1456,6 @@ class Human(guicommon.Object, animation.AnimatedMesh):
         self.callEvent('onChanged', event)
 
     def load(self, filename, update=True, strict=False):
-        from codecs import open
 
         def _compare_versions(mhmVersion,pgmVersion):
             """ Return true if major+minor matches, false if they do not. Ignore patch number. """
@@ -1423,82 +1487,86 @@ class Human(guicommon.Object, animation.AnimatedMesh):
 
         subdivide = False
 
-        f = open(filename, 'rU', encoding="utf-8")
+        with open(filename, 'r', encoding="utf-8") as f:
 
-        for lh in G.app.loadHandlers.values():
-            try:
-                lh(self, ['status', 'started'], strict)
-            except:
-                if strict:
-                    e = sys.exc_info()
-                    raise e[0], e[1], e[2]
-                else:
-                    log.warning("Exception while starting MHM loading.", exc_info=True)
-
-        lines = f.readlines()
-
-        def _load_property(lineData):
-            try:
-                _do_load_property(lineData)
-            except:
-                if strict:
-                    e = sys.exc_info()
-                    raise e[0], e[1], e[2]
-                else:
-                    log.warning("Exception while loading MHM property.", exc_info=True)
-
-        def _do_load_property(lineData):
-            if len(lineData) > 0 and not lineData[0] == '#':
-                if lineData[0] == 'version':
-                    log.message('Version %s', lineData[1])
-                elif lineData[0] == 'tags':
-                    for tag in lineData[1:]:
-                        log.debug('Tag %s', tag)
-                elif lineData[0] == 'modifier':
-                    try:
-                        self.getModifier(lineData[1]).setValue(float(lineData[2]), skipDependencies=True)
-                    except KeyError:
-                        log.warning('Unknown modifier specified in MHM file: %s', lineData[1])
-                elif lineData[0] == 'camera':
-                    rot = map(float, lineData[1:3]) + [0.0]
-                    trans = map(float, lineData[3:6])
-                    zoom = float(lineData[6])
-                    G.app.modelCamera.setRotation(rot)
-                    G.app.modelCamera.translation[:3] = trans[:3]
-                    G.app.modelCamera.setZoomFactor(zoom)
-                elif lineData[0] == 'subdivide':
-                    G.app.selectedHuman._mhm_do_subdivide = lineData[1].lower() in ['true', 'yes']
-                elif lineData[0] in G.app.loadHandlers:
-                    G.app.loadHandlers[lineData[0]](self, lineData, strict)
-                else:
+            for lh in list(G.app.loadHandlers.values()):
+                try:
+                    lh(self, ['status', 'started'], strict)
+                except:
                     if strict:
-                        raise RuntimeError('Unknown property in MHM file: %s' % (lineData, ))
+                        e = sys.exc_info()
+                        raise e[0](e[1]).with_traceback(e[2])
                     else:
-                        log.warning('Unknown property in MHM file: %s', lineData)
+                        log.warning("Exception while starting MHM loading.", exc_info=True)
 
-        version = _get_version(lines)
-        if not _compare_versions(version, getShortVersion(noSub=True)):
-            log.message("MHM file is of version %s, attempting to load with backward compatibility")
-            import compat
-            compat.loadMHM(version, lines, _load_property, strict)
-        else:
-            fprog = Progress(len(lines))
-            for data in lines:
-                lineData = data.strip().split()
-                _load_property(lineData)
-                fprog.step()
+            lines = f.readlines()
 
-        log.debug("Finalizing MHM loading.")
-        for lh in set(G.app.loadHandlers.values()):
-            try:
-                lh(self, ['status', 'finished'], strict)
-            except:
-                if strict:
-                    e = sys.exc_info()
-                    raise e[0], e[1], e[2]
-                else:
-                    log.warning("Exception while finishing MHM loading.", exc_info=True)
-        f.close()
+            def _load_property(lineData):
+                try:
+                    _do_load_property(lineData)
+                except:
+                    if strict:
+                        e = sys.exc_info()
+                        raise e[0](e[1]).with_traceback(e[2])
+                    else:
+                        log.warning("Exception while loading MHM property.", exc_info=True)
+
+            def _do_load_property(lineData):
+                if len(lineData) > 0 and not lineData[0] == '#':
+                    if lineData[0] == 'version':
+                        log.message('Version %s', lineData[1])
+                    elif lineData[0] == 'uuid' and len(lineData) > 1:
+                        self.setUuid(lineData[1])
+                    elif lineData[0] == 'name' and len(lineData) > 1:
+                        self.setName(' '.join(lineData[1:]))
+                        log.debug('Model Name %s' % self.getName())
+                    elif lineData[0] == 'tags' and len(lineData) > 1:
+                        for tag in ' '.join(lineData[1:]).split(';'):
+                            self.addTag(tag)
+                    elif lineData[0] == 'modifier':
+                        try:
+                            self.getModifier(lineData[1]).setValue(float(lineData[2]), skipDependencies=True)
+                        except KeyError:
+                            log.warning('Unknown modifier specified in MHM file: %s', lineData[1])
+                    elif lineData[0] == 'camera':
+                        rot = list(map(float, lineData[1:3])) + [0.0]
+                        trans = list(map(float, lineData[3:6]))
+                        zoom = float(lineData[6])
+                        G.app.modelCamera.setRotation(rot)
+                        G.app.modelCamera.translation[:3] = trans[:3]
+                        G.app.modelCamera.setZoomFactor(zoom)
+                    elif lineData[0] == 'subdivide':
+                        G.app.selectedHuman._mhm_do_subdivide = lineData[1].lower() in ['true', 'yes']
+                    elif lineData[0] in G.app.loadHandlers:
+                        G.app.loadHandlers[lineData[0]](self, lineData, strict)
+                    else:
+                        if strict:
+                            raise RuntimeError('Unknown property in MHM file: %s' % (lineData, ))
+                        else:
+                            log.warning('Unknown property in MHM file: %s', lineData)
+
+            version = _get_version(lines)
+            if not _compare_versions(version, getShortVersion(noSub=True)):
+                log.message("MHM file is of version %s, attempting to load with backward compatibility" % version)
+                import compat
+                compat.loadMHM(version, lines, _load_property, strict)
+            else:
+                fprog = Progress(len(lines))
+                for data in lines:
+                    lineData = data.strip().split()
+                    _load_property(lineData)
+                    fprog.step()
+
+            log.debug("Finalizing MHM loading.")
+            for lh in set(G.app.loadHandlers.values()):
+                try:
+                    lh(self, ['status', 'finished'], strict)
+                except:
+                    if strict:
+                        e = sys.exc_info()
+                        raise e[0](e[1]).with_traceback(e[2])
+                    else:
+                        log.warning("Exception while finishing MHM loading.", exc_info=True)
 
         self.blockEthnicUpdates = False
         self._setEthnicVals()
@@ -1518,54 +1586,57 @@ class Human(guicommon.Object, animation.AnimatedMesh):
         progress(1.0)
         log.message("Done loading MHM file.")
 
-    def save(self, filename, tags):
-        from codecs import open
+    def save(self, filename):
         from progress import Progress
         progress = Progress(len(G.app.saveHandlers))
         event = events3d.HumanEvent(self, 'save')
         event.path = filename
         self.callEvent('onChanging', event)
 
-        f = open(filename, "w", encoding="utf-8")
-        f.write('# Written by MakeHuman %s\n' % getVersionStr())
-        f.write('version %s\n' % getShortVersion(noSub=True))
-        f.write('tags %s\n' % tags)
-        cam_rot = list(G.app.modelCamera.getRotation()[:2])
-        cam_trans = list(G.app.modelCamera.translation[:3])
-        cam_zoom = [G.app.modelCamera.zoomFactor]
-        f.write('camera %s %s %s %s %s %s\n' % tuple(cam_rot + cam_trans + cam_zoom))
+        with open(filename, "w", encoding="utf-8") as f:
+            f.write('# Written by MakeHuman %s\n' % getVersionStr())
+            f.write('version %s\n' % getShortVersion(noSub=True))
+            if self.getUuid():
+                f.write('uuid %s\n' % self.getUuid())
+            if self.getName():
+                f.write('name %s\n' % self.getName())
+            if self.getTags():
+                f.write('tags %s\n' % ';'.join(sorted(self.getTags())))
+            cam_rot = list(G.app.modelCamera.getRotation()[:2])
+            cam_trans = list(G.app.modelCamera.translation[:3])
+            cam_zoom = [G.app.modelCamera.zoomFactor]
+            f.write('camera %s %s %s %s %s %s\n' % tuple(cam_rot + cam_trans + cam_zoom))
 
-        for modifier in self.modifiers:
-            if modifier.getValue() or modifier.isMacro():
-                f.write('modifier %s %f\n' % (modifier.fullName, modifier.getValue()))
+            for modifier in self.modifiers:
+                if modifier.getValue() or modifier.isMacro():
+                    f.write('modifier %s %f\n' % (modifier.fullName, modifier.getValue()))
 
-        class SaveWriter(object):
-            def __init__(self, file_obj):
-                self.f = file_obj
+            class SaveWriter(object):
+                def __init__(self, file_obj):
+                    self.f = file_obj
 
-            def write(self, text):
-                # Ensure that handlers write lines ending with newline character
-                if not text.endswith("\n"):
-                    text = text+"\n"
-                self.f.write(text)
+                def write(self, text):
+                    # Ensure that handlers write lines ending with newline character
+                    if not text.endswith("\n"):
+                        text = text+"\n"
+                    self.f.write(text)
 
-            def writelines(self, text):
-                # Ensure that handlers write lines ending with newline character
-                if not text.endswith("\n"):
-                    text = text+"\n"
-                self.f.writelines(text)
+                def writelines(self, text):
+                    # Ensure that handlers write lines ending with newline character
+                    if not text.endswith("\n"):
+                        text = text+"\n"
+                    self.f.writelines(text)
 
-            def __getattr__(self, attr):
-                return self.f.__getattribute__(attr)
+                def __getattr__(self, attr):
+                    return self.f.__getattribute__(attr)
 
-        f = SaveWriter(f)
+            f = SaveWriter(f)
 
-        for handler in G.app.saveHandlers:
-            handler(self, f)
-            progress.step()
+            for handler in G.app.saveHandlers:
+                handler(self, f)
+                progress.step()
 
-        f.write('subdivide %s' % self.isSubdivided())
+            f.write('subdivide %s' % self.isSubdivided())
 
-        f.close()
         progress(1)
         self.callEvent('onChanged', event)
