@@ -1,4 +1,4 @@
-#!/usr/bin/python2.7
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """
@@ -6,17 +6,17 @@ MakeHuman 3D Transformation functions.
 
 **Project Name:**      MakeHuman
 
-**Product Home Page:** http://www.makehuman.org/
+**Product Home Page:** http://www.makehumancommunity.org/
 
-**Code Home Page:**    https://bitbucket.org/MakeHuman/makehuman/
+**Github Code Home Page:**    https://github.com/makehumancommunity/
 
 **Authors:**           Joel Palmius, Marc Flerackers, Jonas Hauquier
 
-**Copyright(c):**      MakeHuman Team 2001-2017
+**Copyright(c):**      MakeHuman Team 2001-2020
 
 **Licensing:**         AGPL3
 
-    This file is part of MakeHuman (www.makehuman.org).
+    This file is part of MakeHuman Community (www.makehumancommunity.org).
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -94,11 +94,15 @@ class Target(object):
         self.name = name
         self.morphFactor = -1
 
-        try:
-            self._load(self.name)
-        except Exception as e:
+        if self.name:
+            try:
+                self._load(self.name)
+            except Exception as e:
+                self.verts = []
+                log.error('Unable to open %s (%s)', name, e)
+                return
+        else:
             self.verts = []
-            log.error('Unable to open %s (%s)', name, e)
             return
 
         self.faces = obj.getFacesForVertices(self.verts)
@@ -123,7 +127,7 @@ class Target(object):
         import makehuman
         data = []
         license = defaultTargetLicense()
-        with open(name, 'rU') as fd:
+        with open(name, 'r', encoding='utf-8') as fd:
             for line in fd:
                 line = line.strip()
                 if line.startswith('#'):
@@ -223,7 +227,7 @@ class Target(object):
                 np.save(lname, license)
                 return iname, vname, lname
             return iname, vname, None
-        except StandardError, _:
+        except Exception as _:
             log.error('error saving %s', name)
 
     def _load(self, name):
@@ -231,7 +235,7 @@ class Target(object):
         logger.debug('loading target %s', name)
         try:
             self._load_binary(name)
-        except StandardError, _:
+        except Exception as _:
             self._load_text(name)
         logger.debug('loaded target %s', name)
 
@@ -451,9 +455,14 @@ def saveTranslationTarget(obj, targetPath, groupToSave=None, epsilon=0.001):
     vertsToSave = vertsToSave[valid]
     nVertsExported = len(vertsToSave)
 
+    license_str = str(defaultTargetLicense()).split('\n')
+    license_str.append('basemesh hm08')
+    license_str = '\n'.join(['# ' + s for s in license_str])
+
     try:
-        with open(targetPath, 'w') as fileDescriptor:
-            for i in xrange(nVertsExported):
+        with open(targetPath, 'w', encoding='utf-8') as fileDescriptor:
+            fileDescriptor.write('%s\n\n\n' % license_str)
+            for i in range(nVertsExported):
                 fileDescriptor.write('%d %f %f %f\n' % (vertsToSave[i], delta[i,0], delta[i,1], delta[i,2]))
 
         if nVertsExported == 0:
@@ -497,4 +506,4 @@ def defaultTargetLicense():
     import makehuman
     return makehuman.getAssetLicense( {"license": "AGPL3",
                                        "author": "MakeHuman",
-                                       "copyright": "2016 Data Collection AB, Joel Palmius, Jonas Hauquier"} )
+                                       "copyright": "2020 Data Collection AB, Joel Palmius, Jonas Hauquier"} )
