@@ -1,20 +1,20 @@
-#!/usr/bin/python2.7
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """ 
 **Project Name:**      MakeHuman
 
-**Product Home Page:** http://www.makehuman.org/
+**Product Home Page:** http://www.makehumancommunity.org/
 
-**Code Home Page:**    https://bitbucket.org/MakeHuman/makehuman/
+**Github Code Home Page:**    https://github.com/makehumancommunity/
 
 **Authors:**           Marc Flerackers, Glynn Clements, Jonas Hauquier
 
-**Copyright(c):**      MakeHuman Team 2001-2017
+**Copyright(c):**      MakeHuman Team 2001-2020
 
 **Licensing:**         AGPL3
 
-    This file is part of MakeHuman (www.makehuman.org).
+    This file is part of MakeHuman Community (www.makehumancommunity.org).
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -42,6 +42,7 @@ import numpy as np
 import unique # Bugfix for numpy.unique on older numpy versions
 
 import material
+import log
 
 class FaceGroup(object):
     """
@@ -301,15 +302,15 @@ class Object3D(object):
 
         # Filter out and remap masked faces
         fvert = self.fvert[self.face_mask]
-        for i in xrange(self.vertsPerPrimitive):
+        for i in range(self.vertsPerPrimitive):
             fvert[:,i] = other._inverse_parent_map[fvert[:,i]]
 
         # Filter out and remap unused UVs
         fuvs = self.fuvs[self.face_mask]
         uv_idx = np.unique(fuvs.reshape(-1))
         inverse_uv_idx = - np.ones(self.texco.shape[0], dtype=np.int32)
-        inverse_uv_idx[uv_idx] = np.arange(self.texco.shape[0], dtype=np.int32)
-        for i in xrange(self.vertsPerPrimitive):
+        inverse_uv_idx[uv_idx] = np.arange(uv_idx.shape[0], dtype=np.int32)
+        for i in range(self.vertsPerPrimitive):
             fuvs[:,i] = inverse_uv_idx[fuvs[:,i]]
 
         other.setUVs(self.texco[uv_idx])
@@ -629,7 +630,14 @@ class Object3D(object):
         self.group = np.zeros(nfaces, dtype=np.uint16)
         self.face_mask = np.ones(nfaces, dtype=bool)
 
+        # preset number of vertices for all export formats to 4
+        self.vertsPerFaceForExport = 4
         if nfaces != 0:
+
+            # evaluate if we have 3 or 4 vertices per face, do it here so it works for npz or obj file
+            if len(verts[0]) == 4 and verts[0][0] == verts[0][3]:
+                    self.vertsPerFaceForExport = 3
+
             self.fvert[...] = verts
             if uvs is not None:
                 self.fuvs[...] = uvs
@@ -639,7 +647,7 @@ class Object3D(object):
         self.has_uv = uvs is not None
 
         if not skipUpdate:
-            self._update_faces()
+            self._update_faces(resize=True)
 
     def changeFaceMask(self, mask, indices = None):
         if indices is None:
@@ -686,30 +694,81 @@ class Object3D(object):
             self._inverse_vmap = originalToUnweldedMap
         return self._inverse_vmap
 
-    def _update_faces(self):
-        # Construct vface: arrange face indices for same v_idx in different columns
-        # Every row in the vface matrix contains a variable number of valid columns
-        # (the number of valid columns for each row is stored in the nfaces array)
-        map_ = np.argsort(self.fvert.flat)
-        vi = self.fvert.flat[map_]
-        # Map v_idx entries to row numbers of fvert (face_idx)
-        fi = np.mgrid[:self.fvert.shape[0],:self.fvert.shape[1]][0].flat[map_].astype(np.uint32)
-        del map_
-        ix, first = np.unique(vi, return_index=True)
-        n = first[1:] - first[:-1]    # entry-skip count, or the number of occurences of every idx
-        n_last = len(vi) - first[-1]  # Number of occurences of last idx
-        n = np.hstack((n, np.array([n_last])))  # Append last to complete n
+    def _update_faces(self, resize = False):
+        # 
+        # this procedure is only called, when geometry is not taken from npz-file
+        # 
+        # create an 2dim array self.vface for each vertex
+        #     which contains an array of faces, where the vertex belongs to, maximum of the array inside is self.MAX_FACES
+        # 
+        # use self.nfaces as counter for the inner array, it is needed afterwards
+
+        nverts = len(self.coord)
+        self.nfaces = np.zeros(nverts, dtype=np.uint8) # reset the counters to zero
+
+        for idx, vert in enumerate(self.fvert):                             # contains faces with vertex number
+            for i in range (0, min(len(vert), self.vertsPerFaceForExport)): # use minimum of attached vertices, works for less than 3 also
+                vn = vert[i]                                                 # vertex.number
+                if self.nfaces[vn] >= self.MAX_FACES:
+                    log.error("Failed to index faces of mesh %s, you are probably loading a mesh with mixed nb of verts per face (do not mix tris and quads). Or your mesh has too many faces attached to one vertex (the maximum is %s-poles). In the second case, either increase MAX_FACES for this mesh, or improve the mesh topology.", self.name, self.MAX_FACES)
+                    raise RuntimeError('Incompatible mesh topology.')
+                    return
+                self.vface[vn,self.nfaces[vn]] = idx        # add the index of the face to the array, row given by vn, column by counter nfaces[vn]
+                self.nfaces[vn] +=1                         # now increment the counter
+
+        imax = i+1    # just needed later not to be recalculated.
+
+        # in case this function is not called from catmull-clark function resize the self.vface to a minimum
+        if resize is True:
+            # now lets recalculate the maximum number of faces belonging to a plane
+            newmax = np.max(self.nfaces)
+            if newmax > 2:                  # Avoid the information when function calculates internal objects not to confuse users
+                log.debug ("Calculated maximum number of faces for one vertex: %d", newmax)
+
+            # unfortunately catmull-clark expects maxpoles and not maxfaces, so we need 
+            # also to calculate max-poles
+            # 
+            # for each vertex we have to check all attached faces, these faces are already collected in self.vface
+            #       when we check all the faces we will collect the neighbors using a modulo function, the vertices are
+            #       entered clockwise ... modulo allows us to use neighbor of 1 and 3 when index is 0
+            #       if not already existent, the neighbors are added to an array and the maximum size of this array will be maxpole
+            #
+            maxpole = 0
+            for vn, row in enumerate(self.vface):
+                noticed = np.zeros(self.MAX_FACES * 4, dtype = int)
+                m = 0
+                for j in range (0, self.nfaces[vn]):
+                    # collect neighbors
+                    face = row[j]
+                    for ix,v2 in enumerate(self.fvert[face]):
+                        if (v2 == vn):
+                            ln = self.fvert[face][(ix-1) %imax]
+                            rn = self.fvert[face][(ix+1) %imax]
+                            if ln not in noticed:
+                                noticed[m] = ln
+                                m += 1
+                            if rn not in noticed:
+                                noticed[m] = rn
+                                m += 1
+                            break
+                if m > maxpole:
+                    maxpole = m
+
+            if maxpole > 2:                  # Avoid the information when function calculates internal objects not to confuse users
+                log.debug ("Calculated maximum number of poles for one vertex: %d", maxpole)
+
+            if newmax < maxpole:
+                newmax = maxpole
+
+            # keep in mind that a maximum number of neighboring faces lower than 4 will crash subdiv (try to load a cube)
+            if newmax < 4: 
+                newmax = 4
+
+            if newmax != self.MAX_FACES:
+                # it is different so resize vface
+                self.vface = np.delete (self.vface, np.s_[newmax::], 1)
+                self.MAX_FACES = newmax
 
-        # Store number of valid columns per line in vface
-        self.nfaces[ix] = n.astype(np.uint8)
-        try:
-            for i in xrange(len(ix)):
-                # Unfortunately these type of slices require a python loop
-                self.vface[ix[i],:n[i]] = fi[first[i]:][:n[i]]
-        except Exception as e:
-            import log
-            log.error("Failed to index faces of mesh %s, you are probably loading a mesh with mixed nb of verts per face (do not mix tris and quads). Or your mesh has too many faces attached to one vertex (the maximum is %s-poles). In the second case, either increase MAX_FACES for this mesh, or improve the mesh topology. Original error message: (%s) %s", self.name, self.MAX_FACES, type(e), format(str(e)))
-            raise RuntimeError('Incompatible mesh topology.')
 
     def getVertexWeights(self, parentWeights):
         """
@@ -731,7 +790,7 @@ class Object3D(object):
 
         from collections import OrderedDict
         weights = OrderedDict()
-        for bname, (verts,wghts) in parentWeights.data.items():
+        for bname, (verts,wghts) in list(parentWeights.data.items()):
             vgroup = []
             empty = True
             for (v,wt) in zip(verts,wghts):
@@ -1039,7 +1098,7 @@ class Object3D(object):
         """
         The names of the facegroups available on this mesh.
         """
-        return self._groups_rev.keys()
+        return list(self._groups_rev.keys())
 
     def getGroupMaskForGroups(self, groupNames):
         groups = np.zeros(len(self._faceGroups), dtype=bool)
