@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.7
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """
@@ -6,17 +6,17 @@ MakeHuman python entry-point.
 
 **Project Name:**      MakeHuman
 
-**Product Home Page:** http://www.makehuman.org/
+**Product Home Page:** http://www.makehumancommunity.org/
 
-**Code Home Page:**    https://bitbucket.org/MakeHuman/makehuman/
+**Code Home Page:**    https://github.com/makehumancommunity/
 
 **Authors:**           Glynn Clements, Jonas Hauquier, Joel Palmius
 
-**Copyright(c):**      MakeHuman Team 2001-2017
+**Copyright(c):**      MakeHuman Team 2001-2020
 
 **Licensing:**         AGPL3 
 
-    This file is part of MakeHuman (www.makehuman.org).
+    This file is part of MakeHuman Community (www.makehumancommunity.org).
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -37,25 +37,23 @@ Abstract
 This file starts the MakeHuman python application.
 """
 
-from __future__ import absolute_import  # Fix 'from . import x' statements on python 2.6
 import sys
-reload(sys)
-sys.setdefaultencoding('utf-8')
-
 import os
 import re
 import subprocess
 
+from PyQt5 import QtCore
+
 ## Version information #########################################################
-__version__ = "1.1.1"                   # Major, minor and patch version number
+__version__ = "1.2.0"                   # Major, minor and patch version number
 release = False                         # False for nightly
-versionSub = ""                         # Short version description
+versionSub = "final"                    # Short version description
 meshVersion = "hm08"                    # Version identifier of the basemesh
 ################################################################################
 
 __author__ = "Jonas Hauquier, Joel Palmius, Glynn Clements, Thomas Larsson et al."
-__copyright__ = "Copyright 2017 Data Collection AB and listed authors"
-__credits__ = ["See http://www.makehuman.org/halloffame"]
+__copyright__ = "Copyright 2020 Data Collection AB and listed authors"
+__credits__ = ["See http://www.makehumancommunity.org/halloffame"]
 __license__ = "AGPLv3"
 __maintainer__ = "Joel Palmius, Jonas Hauquier"
 __status__ = "Production" if release else "Development"
@@ -67,6 +65,9 @@ def getVersionDigitsStr():
     """
     return __version__
 
+def getVersionSubStr():
+    return versionSub
+
 def _versionStr():
     if versionSub:
         return getVersionDigitsStr() + " " + versionSub
@@ -102,12 +103,9 @@ def getVersionStr(verbose=True, full=False):
     if isRelease() and not full:
         return _versionStr()
     else:
-        if 'HGREVISION' not in os.environ:
-            get_hg_revision()
-        result = _versionStr() + " (r%s %s)" % (os.environ['HGREVISION'], os.environ['HGNODEID'])
-        if verbose:
-            result += (" [%s]" % os.environ['HGREVISION_SOURCE'])
-        return result
+        from mhversion import MHVersion
+        mhv = MHVersion()
+        return mhv.currentBranch + ":" + mhv.currentShortCommit
 
 def getShortVersion(noSub=False):
     """
@@ -124,45 +122,6 @@ def getBasemeshVersion():
     """
     return meshVersion
 
-def unicode(msg, *args, **kwargs):
-    """
-    Override default unicode constructor to try and resolve some issues with
-    mismatched string codecs.
-    Perhaps this is overkill, but better safe than sorry.
-    """
-    try:
-        # First attempt the builtin unicode() function without interference
-        return __builtins__.unicode(msg, *args, **kwargs)
-    except:
-        pass
-    try:
-        # In case msg is an exception, attempt to decode its message
-        return unicode(msg.message)
-    except:
-        pass
-    try:
-        # Try decoding as utf-8 bytestring
-        return __builtins__.unicode(msg, encoding="utf-8")
-    except:
-        pass
-    try:
-        # Try guessing system default encoding and decode as such
-        import locale
-        return __builtins__.unicode(msg, encoding=locale.getpreferredencoding())
-    except:
-        pass
-    try:
-        # Attempt using filesystem encoding
-        import sys
-        return __builtins__.unicode(msg, encoding=sys.getfilesystemencoding())
-    except:
-        pass
-    try:
-        # Attempt to decode object's __str__ into unicode
-        return str(msg).decode("utf-8", errors="replace")
-    except:
-        pass
-    return u"unable to encode message"
 
 def getCwd():
     """
@@ -175,186 +134,12 @@ def getCwd():
     else:
         return os.path.dirname(os.path.realpath(__file__))
 
-def getHgRoot(subpath=''):
-    cwd = getCwd()
-    return os.path.realpath(os.path.join(cwd, '..', subpath))
-
-def get_revision_hg_info():
-    # Return local revision number of hg parent
-    hgRoot = getHgRoot()
-    output = subprocess.Popen(["hg","-q","parents","--template","{rev}:{node|short}"], stdout=subprocess.PIPE, stderr=sys.stderr, cwd=hgRoot).communicate()[0]
-    output = output.strip().split(':')
-    rev = output[0].strip().replace('+', '')
-    revid = output[1].strip().replace('+', '')
-    try:
-        branch = subprocess.Popen(["hg","-q","branch"], stdout=subprocess.PIPE, stderr=sys.stderr, cwd=hgRoot).communicate()[0].replace('\n','').strip()
-    except:
-        branch = None
-    return (rev, revid, branch)
-
-def get_revision_dirstate_parent(folder=None):
-    # First fallback: try to parse the dirstate file in .hg manually
-    import binascii
-
-    dirstatefile = open(getHgRoot('.hg/dirstate'), 'rb')
-    st = dirstatefile.read(40)
-    dirstatefile.close()
-    l = len(st)
-    if l == 40:
-        nodeid = binascii.hexlify(st)[:20]
-        nodeid_short = nodeid[:12]
-    elif l > 0 and l < 40:
-        raise RuntimeError('Hg working directory state appears damaged!')
-
-    # Build mapping of nodeid to local revision number
-    node_rev_map = dict()
-    revlogfile = open(getHgRoot('.hg/store/00changelog.i'), 'rb')
-    st = revlogfile.read(32)
-
-    rev_idx = 0
-    while st:
-        st = revlogfile.read(10)
-        if st:
-            _nodeid = binascii.hexlify(st)
-            node_rev_map[_nodeid] = rev_idx
-
-        rev_idx += 1
-        st = revlogfile.read(54)
-
-    revlogfile.close()
-    if nodeid not in node_rev_map:
-        raise RuntimeError("Failed to lookup local revision number for node %s" % nodeid)
-    rev = node_rev_map[nodeid]
-    return (str(rev), nodeid_short)
-
-
-def get_revision_cache_tip(folder=None):
-    # Second fallback: try to parse the cache file in .hg manually
-    # Retrieves revision of tip, which might not actually be the working dir parent revision
-    cachefile = open(getHgRoot('.hg/cache/tags'), 'r')
-    for line in iter(cachefile):
-        if line == "\n":
-            break
-        line = line.split()
-        rev = int(line[0].strip())
-        nodeid = line[1].strip()
-        nodeid_short = nodeid[:12]
-        # Tip is at the top of the file
-        return (str(rev), nodeid_short)
-    raise RuntimeError("No tip revision found in tags cache file")
-
-def get_revision_hglib():
-    # The following only works if python-hglib is installed.
-    import hglib
-    hgclient = hglib.open(getHgRoot())
-    parent = hgclient.parents()[0]
-    branch = hgclient.branch()
-    return (parent.rev.replace('+',''), parent.node[:12], branch)
-
-def get_revision_file():
-    # Default fallback to use if we can't figure out HG revision in any other
-    # way: Use this file's hg revision.
-    pattern = re.compile(r'[^0-9]')
-    return pattern.sub("", "$Revision: 6893 $")
-
-def get_hg_revision_1():
-    """
-    Retrieve (local) revision number and short nodeId of current working dir
-    parent.
-    """
-    hgrev = None
-
-    try:
-        hgrev = get_revision_hg_info()
-        os.environ['HGREVISION_SOURCE'] = "hg parents command"
-        os.environ['HGREVISION'] = str(hgrev[0])
-        os.environ['HGNODEID'] = str(hgrev[1])
-        if hgrev[2]:
-            os.environ['HGBRANCH'] = hgrev[2]
-        return hgrev
-    except Exception as e:
-        print >> sys.stderr,  u"NOTICE: Failed to get hg version number from command line: " + format(unicode(e)) + u" (This is just a head's up, not a critical error)"
-
-    try:
-        hgrev = get_revision_hglib()
-        os.environ['HGREVISION_SOURCE'] = "python-hglib"
-        os.environ['HGREVISION'] = str(hgrev[0])
-        os.environ['HGNODEID'] = str(hgrev[1])
-        if hgrev[2]:
-            os.environ['HGBRANCH'] = hgrev[2]
-        return hgrev
-    except Exception as e:
-        print >> sys.stderr,  u"NOTICE: Failed to get hg version number using hglib: " + format(unicode(e)) + u" (This is just a head's up, not a critical error)"
-
-    try:
-        hgrev = get_revision_dirstate_parent()
-        os.environ['HGREVISION_SOURCE'] = ".hg dirstate file"
-        os.environ['HGREVISION'] = str(hgrev[0])
-        os.environ['HGNODEID'] = str(hgrev[1])
-        return hgrev
-    except Exception as e:
-        print >> sys.stderr,  u"NOTICE: Failed to get hg parent version from dirstate file: " + format(unicode(e)) + u" (This is just a head's up, not a critical error)"
-
-    try:
-        hgrev = get_revision_cache_tip()
-        os.environ['HGREVISION_SOURCE'] = ".hg cache file tip"
-        os.environ['HGREVISION'] = str(hgrev[0])
-        os.environ['HGNODEID'] = str(hgrev[1])
-        return hgrev
-    except Exception as e:
-        print >> sys.stderr,  u"NOTICE: Failed to get hg tip version from cache file: " + format(unicode(e)) + u" (This is just a head's up, not a critical error)"
-
-    #TODO Disabled this fallback for now, it's possible to do this using the hg keyword extension, but not recommended and this metric was never really reliable (it only caused more confusion)
-    '''
-    print >> sys.stderr,  "NOTICE: Using HG rev from file stamp. This is likely outdated, so the number in the title bar might be off by a few commits."
-    hgrev = get_revision_file()
-    os.environ['HGREVISION_SOURCE'] = "approximated from file stamp"
-    os.environ['HGREVISION'] = hgrev[0]
-    os.environ['HGNODEID'] = hgrev[1]
-    return hgrev
-    '''
-
-    if hgrev is None:
-        rev = "?"
-        revid = "UNKNOWN"
-        hgrev = (rev, revid)
-    else:
-        rev, revid = hgrev
-    os.environ['HGREVISION_SOURCE'] = "none found"
-    os.environ['HGREVISION'] = str(rev)
-    os.environ['HGNODEID'] = str(revid)
-
-    return hgrev
-
-def get_hg_revision():
-    # Use the data/VERSION file if it exists. This is created and managed by build scripts
-    import getpath
-    versionFile = getpath.getSysDataPath("VERSION")
-    if os.path.exists(versionFile):
-        version_ = open(versionFile).read().strip()
-        print >> sys.stderr,  u"data/VERSION file detected using value from version file: %s" % version_
-        os.environ['HGREVISION'] = str(version_.split(':')[0])
-        os.environ['HGNODEID'] = str(version_.split(':')[1])
-        os.environ['HGREVISION_SOURCE'] = "data/VERSION static revision data"
-    elif not isBuild():
-        print >> sys.stderr,  u"NO VERSION file detected, retrieving revision info from HG"
-        # Set HG rev in environment so it can be used elsewhere
-        hgrev = get_hg_revision_1()
-        print >> sys.stderr,  u"Detected HG revision: r%s (%s)" % (hgrev[0], hgrev[1])
-    else:
-        # Don't bother trying to retrieve HG info for a build release, there should be a data/VERSION file
-        os.environ['HGREVISION'] = ""
-        os.environ['HGNODEID'] = ""
-        os.environ['HGREVISION_SOURCE'] = "skipped for build"
-
-    return (os.environ['HGREVISION'], os.environ['HGNODEID'])
-
 def set_sys_path():
     """
     Append local module folders to python search path.
     """
     #[BAL 07/11/2013] make sure we're in the right directory
-    if sys.platform != 'darwin': # Causes issues with py2app builds on MAC
+    if not sys.platform.startswith('darwin'): # Causes issues with py2app builds on MAC
         os.chdir(getCwd())
     syspath = ["./", "./lib", "./apps", "./shared", "./apps/gui","./core"]
     syspath.extend(sys.path)
@@ -373,7 +158,7 @@ def get_platform_paths():
 
     home = getpath.getPath()
 
-    if sys.platform == 'win32':
+    if sys.platform.startswith('win'):
         stdout_filename = os.path.join(home, "python_out.txt")
         stderr_filename = os.path.join(home, "python_err.txt")
 
@@ -382,7 +167,6 @@ def get_platform_paths():
         stderr_filename = os.path.join(home, "makehuman-error.txt")
 
 def redirect_standard_streams():
-    from codecs import open
     import locale
     encoding = locale.getpreferredencoding()
     if stdout_filename:
@@ -405,6 +189,9 @@ def make_user_dir():
     userDataDir = getpath.getPath('data')
     if not os.path.isdir(userDataDir):
         os.makedirs(userDataDir)
+    userPluginsDir = getpath.getPath('plugins')
+    if not os.path.isdir(userPluginsDir):
+        os.makedirs(userPluginsDir)
 
 def init_logging():
     import log
@@ -416,7 +203,7 @@ def debug_dump():
         import debugdump
         debugdump.dump.reset()
     except debugdump.DependencyError as e:
-        print >> sys.stderr,  u"Dependency error: " + format(unicode(e))
+        print("Dependency error: " + format(str(e)), file=sys.stderr)
         import log
         log.error("Dependency error: %s", e)
         sys.exit(-1)
@@ -439,7 +226,7 @@ def parse_arguments():
     parser.add_argument("--debugopengl", action="store_true", help="enable OpenGL error checking and logging (slow)")
     parser.add_argument("--fullloggingopengl", action="store_true", help="log all OpenGL calls (very slow)")
     parser.add_argument("--debugnumpy", action="store_true", help="enable numpy runtime error messages")
-    parser.add_argument("--home-location", action="store", help="set alternative home path")
+    
     if not isRelease():
         parser.add_argument("-t", "--runtests", action="store_true", help="run test suite (for developers)")
 
@@ -449,21 +236,21 @@ def parse_arguments():
     argOptions = vars(parser.parse_args())
 
     if argOptions.get('license', False):
-        print "\n" + getCopyrightMessage() + "\n"
+        print("\n" + getCopyrightMessage() + "\n")
         sys.exit(0)
 
     return argOptions
 
 def getCopyrightMessage(short=False):
     if short:
-        return """MakeHuman Copyright (C) 2001-2017 http://www.makehuman.org
+        return """MakeHuman Copyright (C) 2001-2020 http://www.makehumancommunity.org
 This program comes with ABSOLUTELY NO WARRANTY.
 This is free software, and you are welcome to redistribute it
 under certain conditions. For details use the option --license"""
 
     return """Makehuman is a completely free, open source, innovative and 
 professional software for the modelling of 3-Dimensional humanoid characters
-Copyright (C) 2001-2017  www.makehuman.org
+Copyright (C) 2001-2020  www.makehumancommunity.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -478,16 +265,13 @@ GNU Affero General Public License for more details.
 You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-The MakeHuman source and data are released under the AGPL license.
-This also includes everything that is exported from or by MakeHuman. 
-However, respecting a set of conditions (which are explained in section 
-C of license.txt), you are allowed to instead use the CC0 license 
-for exports. 
+The MakeHuman source code is released under the AGPL license.
+The graphical assets bundled with MakeHuman have been released as CC0.
 
 A human readable explanation of the license terms can be found via
 the MakeHuman home page:
 
-    http://www.makehuman.org
+    http://www.makehumancommunity.org
 
 Licenses for dependencies are included in the licenses folder.
 
@@ -506,22 +290,16 @@ forums at:
 
 The forums is also where you can contact the MakeHuman team. 
 
-Bugs can be reported on the project's bug tracker:
+Bugs can be reported on the project's bug tracker(s):
 
-    http://bugtracker.makehumancommunity.org
+    http://www.makehumancommunity.org/content/bugtracker.html
 """
 
 
 class LicenseInfo(object):
     """
     License information for MakeHuman assets.
-    If no properties are changed, the license object retrieved specifies
-    the licensing information that applies to the assets included in the
-    official MakeHuman release.
-    We consider assets to be the basemesh, targets, proxy definitions and their
-    fitting data, in general all the files in the data folder with the exclusion
-    of the data in the form as retained by the official exporters to which the
-    CC0 exception clause applies.
+    Assets bundled with the official MakeHuman binary have been released as CC0.
     Assets created by third parties can be bound to different licensing conditions,
     which is why properties can be set as a dict of format:
         {"author": ..., "license": ..., "copyright": ..., "homepage": ...}
@@ -532,9 +310,9 @@ class LicenseInfo(object):
         user-created assets.
         """
         self.author = "MakeHuman Team"
-        self.license = "AGPL3"
-        self.homepage = "http://www.makehuman.org"
-        self.copyright = "(c) MakeHuman.org 2001-2017"
+        self.license = "CC0"
+        self.homepage = "http://www.makehumancommunity.org"
+        self.copyright = "(c) www.makehumancommunity.org 2001-2020"
         self._keys = ["author", "license", "copyright", "homepage"]
         self._customized = False
 
@@ -574,7 +352,7 @@ Homepage: %s""" % (self.author, self.license, self.copyright, self.homepage)
         return dict( [(pname, getattr(self, pname)) for pname in self._keys] )
 
     def fromDict(self, propDict):
-        for prop,val in propDict.items():
+        for prop,val in list(propDict.items()):
             self.setProperty(prop, val)
         return self
 
@@ -612,13 +390,13 @@ Homepage: %s""" % (self.author, self.license, self.copyright, self.homepage)
             import numpy as np
             text = ''
             index = []
-            for key,value in stringDict.items():
+            for key,value in list(stringDict.items()):
                 index.append(len(key))
                 index.append(len(value))
                 text += key + value
             text = np.fromstring(text, dtype='S1')
             index = np.array(index, dtype=np.uint32)
-            return text, index
+            return np.array([text, index], dtype=object)
 
         return _packStringDict(self.asDict())
 
@@ -631,7 +409,7 @@ Homepage: %s""" % (self.author, self.license, self.copyright, self.homepage)
                 l_val = index[i+1]
 
                 key = text[last:last+l_key].tostring()
-                val = text[last+l_key:last+l_key+l_val].tostring()
+                val = str(text[last+l_key:last+l_key+l_val].tostring(), 'utf8')
                 stringDict[key] = val
 
                 last += (l_key + l_val)
@@ -645,13 +423,7 @@ Homepage: %s""" % (self.author, self.license, self.copyright, self.homepage)
 def getAssetLicense(properties=None):
     """
     Retrieve the license information for MakeHuman assets.
-    If no custom properties are specified, the license object retrieved specifies
-    the licensing information that applies to the assets included in the
-    official MakeHuman release.
-    We consider assets to be the basemesh, targets, proxy definitions and their
-    fitting data, in general all the files in the data folder with the exclusion
-    of the data in the form as retained by the official exporters to which the
-    CC0 exception clause applies.
+    Assets bundled with the official MakeHuman binary have been released as CC0.
     Assets created by third parties can be bound to different licensing conditions,
     which is why properties can be set as a dict of format:
         {"author": ..., "license": ..., "copyright": ..., "homepage": ...}
@@ -700,11 +472,11 @@ def getCredits(richtext=False):
     else:
         result = ''
     return result + '''The list of people that made this software can be found at our website at 
-http://www.makehuman.org/halloffame'''
+http://www.makehumancommunity.org/halloffame'''
 
 def getSoftwareLicense(richtext=False):
     import getpath
-    from codecs import open
+    from io import open
     lfile = getpath.getSysPath('license.txt')
     if not os.path.isfile(lfile):
         if richtext:
@@ -722,7 +494,6 @@ def getSoftwareLicense(richtext=False):
 
 def getThirdPartyLicenses(richtext=False):
     import getpath
-    from codecs import open
     from collections import OrderedDict
     def _title(name, url, license):
         if richtext:
@@ -755,13 +526,12 @@ makes use of.\n"""
     license_folder = getpath.getSysPath('licenses')
     if not os.path.isdir(license_folder):
         return result + _error("Error: external licenses folder is not found, this is an incomplete MakeHuman distribution!")
-    external_licenses = [ ("PyQt4", ("pyQt4-license.txt", "http://www.riverbankcomputing.co.uk", "GPLv3")),
-                          ("Qt4", ("qt4-license.txt", "http://www.qt-project.org", "LGPLv2.1")),
+    external_licenses = [ ("PyQt5", ("pyQt5-license.txt", "http://www.riverbankcomputing.com", "GPLv3")),
+                          ("Qt5", ("qt5-license.txt", "http://www.qt.io", "GPLv3")),
                           ("Numpy", ("numpy-license.txt", "http://www.numpy.org", "BSD (3-clause)")),
                           ("PyOpenGL", ("pyOpenGL-license.txt", "http://pyopengl.sourceforge.net", "BSD (3-clause)")),
                           ("Transformations", ("transformations-license.txt", "http://www.lfd.uci.edu/~gohlke/", "BSD (3-clause)")),
-                          ("pyFBX", ("pyFbx-license.txt", "http://wiki.blender.org/index.php/Extensions:2.6/Py/Scripts/Import-Export/Autodesk_FBX", "GPLv2")),
-                          ("Python hglib", ("hglib-license.txt", "http://mercurial.selenic.com/wiki/PythonHglib", "MIT"))
+                          ("pyFBX", ("pyFbx-license.txt", "http://wiki.blender.org/index.php/Extensions:2.6/Py/Scripts/Import-Export/Autodesk_FBX", "GPLv2"))
                         ]
     external_licenses = OrderedDict(external_licenses)
 
@@ -772,9 +542,9 @@ makes use of.\n"""
         if not os.path.isfile(lfile):
             result += "\n%s\n" % _error("Error: License file %s is not found, this is an incomplete MakeHuman distribution!" % lfile)
             continue
-        f = open(lfile, encoding='utf-8')
-        text = f.read()
-        f.close()
+        with open(lfile, encoding='utf-8') as f:
+            text = f.read()
+
         text = _wordwrap(text)
         result += "\n%s\n" % _block(text)
 
@@ -782,27 +552,23 @@ makes use of.\n"""
 
 
 def main():
-    print getCopyrightMessage(short=True) + "\n"
-    
-    
+    print(getCopyrightMessage(short=True) + "\n")
+
     try:
         set_sys_path()
-        args = parse_arguments()
-        from core import G
-        G.args = args
         make_user_dir()
         get_platform_paths()
         redirect_standard_streams()
-        get_hg_revision()
         os.environ['MH_VERSION'] = getVersionStr()
         os.environ['MH_SHORT_VERSION'] = getShortVersion()
         os.environ['MH_MESH_VERSION'] = getBasemeshVersion()
+        args = parse_arguments()
         init_logging()
     except Exception as e:
-        print >> sys.stderr,  "error: " + format(unicode(e))
+        print("error: " + format(str(e)), file=sys.stderr)
         import traceback
         bt = traceback.format_exc()
-        print >> sys.stderr, bt
+        print(bt, file=sys.stderr)
         return
 
     # Pass release info to debug dump using environment variables
@@ -810,10 +576,11 @@ def main():
     os.environ['MH_RELEASE'] = "Yes" if isRelease() else "No"
 
     debug_dump()
-    
-           
+    from core import G
+    G.args = args
+
     # Set numpy properties
-    if not G.args.get('debugnumpy', False):
+    if not args.get('debugnumpy', False):
         import numpy
         # Suppress runtime errors
         numpy.seterr(all = 'ignore')
