@@ -1,20 +1,20 @@
-#!/usr/bin/python2.7
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """
 **Project Name:**      MakeHuman
 
-**Product Home Page:** http://www.makehuman.org/
+**Product Home Page:** http://www.makehumancommunity.org/
 
-**Code Home Page:**    https://bitbucket.org/MakeHuman/makehuman/
+**Github Code Home Page:**    https://github.com/makehumancommunity/
 
 **Authors:**           Glynn Clements, Jonas Hauquier
 
-**Copyright(c):**      MakeHuman Team 2001-2017
+**Copyright(c):**      MakeHuman Team 2001-2020
 
 **Licensing:**         AGPL3
 
-    This file is part of MakeHuman (www.makehuman.org).
+    This file is part of MakeHuman (www.makehumancommunity.org).
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -81,6 +81,12 @@ class MaterialEditorTaskView(gui3d.TaskView):
 
         self.materialBox = self.addRightWidget(gui.GroupBox('Material settings'))
 
+        # add a description box to inform about the asset
+        descBox = self.addLeftWidget(gui.GroupBox('Description'))
+        self.descrLbl = descBox.addWidget(gui.TextView(''))
+        self.descrLbl.setSizePolicy(gui.QtWidgets.QSizePolicy.Ignored, gui.QtWidgets.QSizePolicy.Preferred)
+        self.descrLbl.setWordWrap(True)
+
         if not shader.Shader.supported():
             log.notice('Shaders not supported')
             self.shaderList.setEnabled(False)
@@ -89,7 +95,7 @@ class MaterialEditorTaskView(gui3d.TaskView):
 
         @self.shaderList.mhEvent
         def onClicked(item):
-            self.setShader(unicode(item.getUserData()))
+            self.setShader(str(item.getUserData()))
 
         self.loadSaveBox = self.addRightWidget(gui.GroupBox("Material file"))
         self.loadMaterialBtn = self.loadSaveBox.addWidget(gui.BrowseButton(), 0, 0)
@@ -379,9 +385,9 @@ class MaterialEditorTaskView(gui3d.TaskView):
         selectedShader = None
         for name in sorted(shaders):
             item = self.shaderList.addItem(name, data = os.path.join(dir, name))
-            if shaderName and unicode(shaderName) == unicode(item.text):
+            if shaderName and str(shaderName) == str(item.text):
                 selectedShader = item
-                path = unicode(item.getUserData())
+                path = str(item.getUserData())
 
         if mat.shader and not selectedShader:
             # Custom shader selected (not in shader path): add it
@@ -498,6 +504,12 @@ class MaterialEditorTaskView(gui3d.TaskView):
     def reloadMaterial(self):
         obj = self.getSelectedObject()
 
+        # update description
+        self.descrLbl.setText (obj.name + ":\n" +
+                str(len(obj.mesh.coord)) + " vertices\n" +
+                str(len(obj.mesh.fvert)) + " faces\n" +
+                str (obj.mesh.vertsPerFaceForExport) + " vertices per face.")
+
         if shader.Shader.supported():
             self.listShaders(obj.material)
             self.listUniforms(obj.material)
@@ -534,7 +546,7 @@ class ColorValue(gui.GroupBox):
         self.name = name
 
         self.widgets = []
-        for col in xrange(3):
+        for col in range(3):
             child = FloatValue(self, 0)
             self.addWidget(child, 0, col)
             self.widgets.append(child)
@@ -694,9 +706,9 @@ class UniformValue(gui.GroupBox):
             values = np.atleast_2d(values)
         rows, cols = values.shape
         self.widgets = []
-        for row in xrange(rows):
+        for row in range(rows):
             widgets = []
-            for col in xrange(cols):
+            for col in range(cols):
                 child = self.createWidget(values[row,col], row)
                 self.addWidget(child, row, col)
                 widgets.append(child)
@@ -773,7 +785,7 @@ class StringValue(gui.TextEdit):
 
     @property
     def value(self):
-        return unicode(self.text)
+        return str(self.text)
 
     def setValue(self, value):
         self.setText(value)
@@ -823,12 +835,12 @@ class BooleanValue(gui.CheckBox):
     def setValue(self, value):
         self.setChecked(value)
 
-class TextureValue(gui.QtGui.QWidget, gui.Widget):
+class TextureValue(gui.QtWidgets.QWidget, gui.Widget):
     def __init__(self, parent, value, defaultPath = None):
         super(TextureValue, self).__init__()
         self.parent = parent
 
-        self.layout = gui.QtGui.QGridLayout(self)
+        self.layout = gui.QtWidgets.QGridLayout(self)
         self.imageView = gui.ImageView()
         self.browseBtn = gui.BrowseButton()
         self.browseBtn.setFilter("Image Files (*.png *.jpg *.bmp)")
@@ -839,7 +851,7 @@ class TextureValue(gui.QtGui.QWidget, gui.Widget):
         self.value = value
 
         self.setValue(value)
-        if value and isinstance(value, basestring):
+        if value and isinstance(value, str):
             self.browseBtn._path = pathToUnicode(value)
         elif defaultPath:
             self.browseBtn._path = pathToUnicode(defaultPath)
@@ -856,13 +868,13 @@ class TextureValue(gui.QtGui.QWidget, gui.Widget):
         return self._path
 
     def setValue(self, value):
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             self._path = pathToUnicode(value)
         else:
             self._path = None
         if value:
             self.imageView.setImage(value)
-            if isinstance(value, basestring):
+            if isinstance(value, str):
                 self.browseBtn._path = pathToUnicode(value)
         else:
             self.imageView.setImage(mh.getSysDataPath('notfound.thumb'))
