@@ -1,20 +1,20 @@
-#!/usr/bin/python2.7
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """
 **Project Name:**      MakeHuman
 
-**Product Home Page:** http://www.makehuman.org/
+**Product Home Page:** http://www.makehumancommunity.org/
 
-**Code Home Page:**    https://bitbucket.org/MakeHuman/makehuman/
+**Github Code Home Page:**    https://github.com/makehumancommunity/
 
 **Authors:**           Joel Palmius
 
-**Copyright(c):**      MakeHuman Team 2001-2017
+**Copyright(c):**      MakeHuman Team 2001-2020
 
 **Licensing:**         AGPL3
 
-    This file is part of MakeHuman (www.makehuman.org).
+    This file is part of MakeHuman (www.makehumancommunity.org).
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -35,7 +35,6 @@ Abstract
 
 TODO
 """
-
 # We need this for gui controls
 import gui3d
 import mh
@@ -44,9 +43,7 @@ import material
 import gui
 import log
 import os
-from cStringIO import StringIO
 from core import G
-from codecs import open
 
 class ScriptingView(gui3d.TaskView):
 
@@ -56,19 +53,28 @@ class ScriptingView(gui3d.TaskView):
         self.directory = os.getcwd()
         self.filename =  None
 
+        scriptingHome = mh.getPath('scripts')
+        if not os.path.exists(scriptingHome):
+            try:
+                os.mkdir(scriptingHome)
+            except OSError:
+                scriptingHome = mh.getPath()
+
         box = self.addLeftWidget(gui.GroupBox('Script'))
 
         self.scriptText = self.addTopWidget(gui.DocumentEdit())
-        self.scriptText.setText('');
+        self.scriptText.setText('')
 
         self.scriptText.setLineWrapMode(gui.DocumentEdit.NoWrap)
 
         self.loadButton = box.addWidget(gui.BrowseButton(mode='open'), 0, 0)
         self.loadButton.setLabel('Load ...')
-        self.loadButton.directory = mh.getPath()
+        self.loadButton.directory = scriptingHome
         self.saveButton = box.addWidget(gui.BrowseButton(mode='save'), 0, 1)
         self.saveButton.setLabel('Save ...')
-        self.saveButton.directory = mh.getPath()
+        self.saveButton.directory = scriptingHome
+
+
 
         @self.loadButton.mhEvent
         def onClicked(filename):
@@ -76,7 +82,8 @@ class ScriptingView(gui3d.TaskView):
                 return
 
             if(os.path.exists(filename)):
-                contents = open(filename, 'rU', encoding="utf-8").read()
+                with open(filename, 'r', encoding="utf-8") as f:
+                    contents = f.read()
                 self.scriptText.setText(contents)
                 dlg = gui.Dialog()
                 dlg.prompt("Load script","File was loaded in an acceptable manner","OK")
@@ -175,7 +182,7 @@ class ScriptingView(gui3d.TaskView):
                 text = "# incrementingFilename(<file name base>, [file extension], [pad length])\n"
                 text = text + "#\n"
                 text = text + "# This will return a file name containing a numerical component which increases by one for each call.\n"
-                text = text + "# The default file extension is \".png\". The default pad length is 4. For example, the following lines:\n";
+                text = text + "# The default file extension is \".png\". The default pad length is 4. For example, the following lines:\n"
                 text = text + "#\n"
                 text = text + "# print incrementingFilename(\"test\",\".target\",3) + \"\\n\"\n"
                 text = text + "# print incrementingFilename(\"test\",\".target\",3) + \"\\n\"\n"
@@ -351,7 +358,7 @@ class ScriptingView(gui3d.TaskView):
                 text = text + "#\n"
                 text = text + "# Sets the Y position of the model of the model in 3d space, where 0.0 is centered.\n"
                 text = text + "# Note that the depth of the scene is clipped, so if you move the model too far back\n"
-                text = text + "# it will disappear. You will most likely want to use zoom instead of Y position.\n\n";
+                text = text + "# it will disappear. You will most likely want to use zoom instead of Y position.\n\n"
                 text = text + "MHScript.setPositionY(2.0)\n\n"
                 self.scriptText.addText(text)
 
@@ -367,7 +374,7 @@ class ScriptingView(gui3d.TaskView):
                 text = text + "#\n"
                 text = text + "# Modifies Y position of the model of the model in 3d space.\n"
                 text = text + "# Note that the depth of the scene is clipped, so if you move the model too far back\n"
-                text = text + "# it will disappear. You will most likely want to use zoom instead of Y position.\n\n";
+                text = text + "# it will disappear. You will most likely want to use zoom instead of Y position.\n\n"
                 text = text + "MHScript.modifyPositionY(-0.1)\n\n"
                 self.scriptText.addText(text)
 
@@ -485,8 +492,8 @@ class ScriptingExecuteTab(gui3d.TaskView):
 
         @self.execButton.mhEvent
         def onClicked(event):
-            #width = G.windowWidth;
-            #height = G.windowHeight;
+            #width = G.windowWidth
+            #height = G.windowHeight
 
             global MHScript
             global scriptingView
@@ -505,8 +512,8 @@ class ScriptingExecuteTab(gui3d.TaskView):
 
         @self.getButton.mhEvent
         def onClicked(event):
-            width = G.windowWidth;
-            height = G.windowHeight;
+            width = G.windowWidth
+            height = G.windowHeight
             self.widthEdit.setText(str(width))
             self.heightEdit.setText(str(height))
 
@@ -515,12 +522,12 @@ class ScriptingExecuteTab(gui3d.TaskView):
             dlg = gui.Dialog()
 
             desiredWidth = self.widthEdit.getText()
-            if(desiredWidth == None or not desiredWidth.isdigit()):
+            if(desiredWidth is None or not desiredWidth.isdigit()):
                 dlg.prompt("Input error","Width and height must be valid integers","OK")
                 return
 
             desiredHeight = self.heightEdit.getText()
-            if(desiredHeight == None or not desiredHeight.isdigit()):
+            if(desiredHeight is None or not desiredHeight.isdigit()):
                 dlg.prompt("Input error","Width and height must be valid integers","OK")
                 return
 
@@ -539,11 +546,11 @@ class ScriptingExecuteTab(gui3d.TaskView):
             central = qmainwin.centralWidget() 
             cWidth = central.frameSize().width()
             cHeight = central.frameSize().height()
-            width = G.windowWidth;
-            height = G.windowHeight;
+            width = G.windowWidth
+            height = G.windowHeight
 
-            xdiff = desiredWidth - width;
-            ydiff = desiredHeight - height;
+            xdiff = desiredWidth - width
+            ydiff = desiredHeight - height
 
             cWidth = cWidth + xdiff
             cHeight = cHeight + ydiff
@@ -554,7 +561,7 @@ class ScriptingExecuteTab(gui3d.TaskView):
 class Scripting():
     def __init__(self):
         self.human = gui3d.app.selectedHuman
-        self.fileIncrement = 0;
+        self.fileIncrement = 0
         self.modelPath = mh.getPath('models')
         self.cam = G.app.modelCamera
         if(not os.path.exists(self.modelPath)):
@@ -584,28 +591,28 @@ class Scripting():
 
     def screenShot(self,fileName):
         log.message("SCRIPT: screenShot(" + fileName + ")")
-        width = G.windowWidth;
-        height = G.windowHeight;
-        width = width - 3;
-        height = height - 3;
+        width = G.windowWidth
+        height = G.windowHeight
+        width = width - 3
+        height = height - 3
         mh.grabScreen(1,1,width,height,fileName)
 
     def incrementingFilename(self,basename,suffix=".png",width=4):
-        fn = basename;
-        i = width - 1;
+        fn = basename
+        i = width - 1
         self.fileIncrement += 1
         while(i > 0):       
-            power = 10**i;
+            power = 10**i
             if(self.fileIncrement < power):
-                fn = fn + "0";
+                fn = fn + "0"
             i -= 1
         fn = fn + str(self.fileIncrement) + suffix
         return fn        
 
     def printDetailStack(self):
         log.message("SCRIPT: printDetailStack()")
-        for target in self.human.targetsDetailStack.keys():
-            print str(self.human.targetsDetailStack[target]) + "\t" + target
+        for target in list(self.human.targetsDetailStack.keys()):
+            print(str(self.human.targetsDetailStack[target]) + "\t" + target)
 
     def setAge(self,age):
         log.message("SCRIPT: setAge(" + str(age) + ")")
@@ -630,8 +637,8 @@ class Scripting():
     def getModelingParameters(self):
         log.message("SCRIPT: getModelingParameters()")
         modifierNamesList = sorted( self.human.modifierNames )
-        print "Modifier names:"
-        print "\n".join( modifierNamesList )
+        print("Modifier names:")
+        print("\n".join( modifierNamesList ))
 
     def updateModelingParameter(self, parameterName, value):
         log.message("SCRIPT: updateModelingParameter(parameterName, value)")
@@ -642,7 +649,7 @@ class Scripting():
 
     def updateModelingParameters(self, dictOfParameterNameAndValue):
         log.message("SCRIPT: updateModelingParameters("+str(dictOfParameterNameAndValue)+")")
-        for key, value in dictOfParameterNameAndValue.iteritems():
+        for key, value in dictOfParameterNameAndValue.items():
             modifier = self.human.getModifier(key)
             modifier.setValue(value)
         self.human.applyAllTargets()
@@ -773,33 +780,33 @@ class Scripting():
         log.message("SCRIPT: printCameraInfo()")
 
         # TODO update to new camera
-        print "eyeX:\t" + str(self.cam.eyeX)
-        print "eyeY:\t" + str(self.cam.eyeY)
-        print "eyeZ:\t" + str(self.cam.eyeZ)
-        print "focusX:\t" + str(self.cam.focusX)
-        print "focusY:\t" + str(self.cam.focusY)
-        print "focusZ:\t" + str(self.cam.focusZ)
-        print "upX:\t" + str(self.cam.upX)
-        print "upY:\t" + str(self.cam.upY)
-        print "upZ:\t" + str(self.cam.upZ)
+        print("eyeX:\t" + str(self.cam.eyeX))
+        print("eyeY:\t" + str(self.cam.eyeY))
+        print("eyeZ:\t" + str(self.cam.eyeZ))
+        print("focusX:\t" + str(self.cam.focusX))
+        print("focusY:\t" + str(self.cam.focusY))
+        print("focusZ:\t" + str(self.cam.focusZ))
+        print("upX:\t" + str(self.cam.upX))
+        print("upY:\t" + str(self.cam.upY))
+        print("upZ:\t" + str(self.cam.upZ))
 
     def printPositionInfo(self):
         log.message("SCRIPT: printPositionInfo()")
 
-        pos = self.human.getPosition();
+        pos = self.human.getPosition()
 
-        print "posX:\t" + str(pos[0])
-        print "posY:\t" + str(pos[2])
-        print "posZ:\t" + str(pos[1])
+        print("posX:\t" + str(pos[0]))
+        print("posY:\t" + str(pos[2]))
+        print("posZ:\t" + str(pos[1]))
 
     def printRotationInfo(self):
         log.message("SCRIPT: printRotationInfo()")
 
-        rot = self.human.getRotation();
+        rot = self.human.getRotation()
 
-        print "rotX:\t" + str(rot[0])
-        print "rotY:\t" + str(rot[2])
-        print "rotZ:\t" + str(rot[1])
+        print("rotX:\t" + str(rot[0]))
+        print("rotY:\t" + str(rot[2]))
+        print("rotZ:\t" + str(rot[1]))
 
     def getZoom(self):
         log.message("SCRIPT: getZoom()")
@@ -818,7 +825,7 @@ class Scripting():
 MHScript = None
 
 def executeScript(scriptSource):
-    print scriptSource
+    print(scriptSource)
     try:
         exec(scriptSource)
         dlg = gui.Dialog()
