@@ -1,4 +1,4 @@
-#!/usr/bin/python2.7
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """ 
@@ -6,17 +6,17 @@ Renderman Export functions
 
 **Project Name:**      MakeHuman
 
-**Product Home Page:** http://www.makehuman.org/
+**Product Home Page:** http://www.makehumancommunity.org/
 
-**Code Home Page:**    https://bitbucket.org/MakeHuman/makehuman/
+**Github Code Home Page:**    https://github.com/makehumancommunity/
 
 **Authors:**           Marc Flerackers
 
-**Copyright(c):**      MakeHuman Team 2001-2017
+**Copyright(c):**      MakeHuman Team 2001-2020
 
 **Licensing:**         AGPL3
 
-    This file is part of MakeHuman (www.makehuman.org).
+    This file is part of MakeHuman Community (www.makehumancommunity.org).
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -82,9 +82,9 @@ class RMRMaterial:
 
     def writeRibCode(self, file):
         file.write('\t\t%s "%s" '%(self.type,self.name))
-        #print "Writing %s material"%(self.name)
+        #print (Writing %s material"%(self.name))
         for p in self.parameters:
-            #print p.name, p.val
+            #print (p.name, p.val)
             if p.type == "float":
                 file.write('"%s %s" [%f] '%(p.type, p.name, p.val))
             if p.type == "string":
@@ -100,10 +100,10 @@ class RMRMaterial:
                 newParamater = False
                 p.val = val
         if newParamater == True:
-            #print "Setting paramater %s with value %s"%(name, str(val))
+            #print ("Setting paramater %s with value %s"%(name, str(val)))
             self.parameters.append(MaterialParameter(pType, name, val))
             #for p in  self.parameters:
-                #print p.name, p.val
+                #print (p.name, p.val)
 
 
 
@@ -210,8 +210,8 @@ class RMRLight:
 
     def placeShadowCamera(self, ribfile):
         direction = self.lookAt - self.position
-        #print "VIEW",self.lookAt, self.position
-        #print "DIRECTION: ", direction
+        #print ("VIEW",self.lookAt, self.position)
+        #print ("DIRECTION: ", direction)
         self.shadowProjection(ribfile)
         if self.roll:
             self.shadowRotate(ribfile,-self.roll, 0.0, 0.0, 1.0);
@@ -260,7 +260,7 @@ class RMRObject:
 
     def writeRibCode(self, ribPath ):
 
-        #print "ribPath = ", ribPath
+        #print ("ribPath = ", ribPath)
         facesUVvalues = self.meshData.texco #TODO usa direttamente self.
 
         ribObjFile = file(ribPath, 'w')
@@ -314,8 +314,8 @@ class RMRHuman(RMRObject):
             self.hairtexture =  os.path.splitext(os.path.basename(self.human.hairObj.getTexture()))[0]
             self.hairMat = RMRMaterial("hairpoly")
             self.hairMat.parameters.append(MaterialParameter("string", "colortexture", self.hairtexture+".png"))
-            #print "HAIRTEXTURE",  self.hairtexture
-        #print "BASETEXTURE",  self.basetexture
+            #print ("HAIRTEXTURE",  self.hairtexture)
+        #print ("BASETEXTURE",  self.basetexture)
         
         
         
@@ -562,27 +562,27 @@ class RMRScene:
         self.lights = []
         RMRLight.lightCounter = 0
         path = os.path.join(lightsFolderPath,lightFile)
-        fileDescriptor = open(path)
+        with open(path, 'r', encoding='utf-8') as fileDescriptor:
 
-        for data in fileDescriptor:
-            #print data
-            dataList = data.split()
-            fromX = float(dataList[0])
-            fromY = float(dataList[1])
-            fromZ = float(dataList[2])
-            toX = float(dataList[3])
-            toY = float(dataList[4])
-            toZ = float(dataList[5])
-            lIntensity = float(dataList[6])
-            lType = dataList[7]
+            for data in fileDescriptor:
+                #print (data)
+                dataList = data.split()
+                fromX = float(dataList[0])
+                fromY = float(dataList[1])
+                fromZ = float(dataList[2])
+                toX = float(dataList[3])
+                toY = float(dataList[4])
+                toZ = float(dataList[5])
+                lIntensity = float(dataList[6])
+                lType = dataList[7]
 
-            l = RMRLight(self.ribsPath,[fromX, fromY, fromZ], [toX, toY, toZ], intensity = lIntensity, type = lType)
-            if len(dataList) >= 9:
-                l.blur = float(dataList[8])
-            if len(dataList) >= 10:
-                l.coneangle = float(dataList[9])
-            #print l
-            self.lights.append(l)
+                l = RMRLight(self.ribsPath,[fromX, fromY, fromZ], [toX, toY, toZ], intensity = lIntensity, type = lType)
+                if len(dataList) >= 9:
+                    l.blur = float(dataList[8])
+                if len(dataList) >= 10:
+                    l.coneangle = float(dataList[9])
+                #print l
+                self.lights.append(l)
 
 
 
@@ -600,13 +600,13 @@ class RMRScene:
         self.humanCharacter.subObjectsInit()
 
         #if len(self.humanCharacter.subObjects) < 1:
-            #print "Warning: AO calculation on 0 objects"
+            #print ("Warning: AO calculation on 0 objects")
 
         ribfile = file(fName, 'w')
         #if not bakeMode:
-            #print "Writing world"
+            #print( "Writing world")
         for subObj in self.humanCharacter.subObjects:
-            #print "rendering....", subObj.name
+            #print ("rendering....", subObj.name)
             ribPath = os.path.join(self.ribsPath, subObj.name + '.rib')
             ribfile.write('\tAttributeBegin\n')
             subObj.writeRibCode(ribPath)
@@ -624,7 +624,7 @@ class RMRScene:
         #    ribfile.write('\tSurface "null"\n')
         #ribfile.write('\tAttributeEnd\n')
         #else:
-            #print "Writing bake world"
+            #print ("Writing bake world")
             #ribfile.write('\tAttributeBegin\n')
             #ribfile.write('\tSurface "bakelightmap" "string bakefilename" "%s" "string texturename" "%s"\n'%(self.bakeTMPTexture, self.humanCharacter.basetexture+".png"))
             #ribPath = os.path.join(self.ribsPath, 'skin.rib')
@@ -873,14 +873,14 @@ class RenderThread(Thread):
             #for line in renderProc.stdout:
               #pass
               
-            #print "Rendering finished ", renderProc.stdout
+            #print ("Rendering finished ", renderProc.stdout)
         """
             
         """
         for filename, status in self.filenames:
 
             command = '%s "%s"' % ('aqsis -progress -progressformat="progress %f %p %s %S" -v 0', filename)
-            #print command
+            #print (command)
             renderProc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, universal_newlines=True)
 
             mh.callAsync(lambda:self.app.progress(0.0, status))
