@@ -1,20 +1,20 @@
-#!/usr/bin/python2.7
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """
 **Project Name:**      MakeHuman
 
-**Product Home Page:** http://www.makehuman.org/
+**Product Home Page:** http://www.makehumancommunity.org/
 
-**Code Home Page:**    https://bitbucket.org/MakeHuman/makehuman/
+**Github Code Home Page:**    https://github.com/makehumancommunity/
 
 **Authors:**           Thomas Larsson, Jonas Hauquier
 
-**Copyright(c):**      MakeHuman Team 2001-2017
+**Copyright(c):**      MakeHuman Team 2001-2020
 
 **Licensing:**         AGPL3
 
-    This file is part of MakeHuman (www.makehuman.org).
+    This file is part of MakeHuman Community (www.makehumancommunity.org).
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -48,7 +48,6 @@ import makehuman
 import material
 import json
 
-
 #
 #   Proxy types. Loop over simple proxy types to do all proxies.
 #   Some code use lowercase proxy types instead.
@@ -114,6 +113,8 @@ class Proxy:
 
         self.deleteVerts = np.zeros(human.meshData.getVertexCount(), bool)
 
+        self.weightsCache = None
+        self.cacheSkel = None
 
     @property
     def material_file(self):
@@ -159,19 +160,20 @@ class Proxy:
         import files3d
         import guicommon
 
+        obj = False
         mesh = files3d.loadMesh(self.obj_file, maxFaces = self.max_pole)
         if not mesh:
             log.error("Failed to load %s", self.obj_file)
-
-        mesh.priority = self.z_depth           # Set render order
-        mesh.setCameraProjection(0)             # Set to model camera
-
-        obj = self.object = guicommon.Object(mesh, human.getPosition())
-        obj.proxy = self
-        obj.material = self.material
-        obj.setRotation(human.getRotation())
-        obj.setSolid(human.solid)    # Set to wireframe if human is in wireframe
-        # TODO perhaps other properties should be copied from human to object, such as subdivision state. For other hints, and duplicate code, see guicommon Object.setProxy()
+        else:
+            mesh.priority = self.z_depth           # Set render order
+            mesh.setCameraProjection(0)             # Set to model camera
+        
+            obj = self.object = guicommon.Object(mesh, human.getPosition())
+            obj.proxy = self
+            obj.material = self.material
+            obj.setRotation(human.getRotation())
+            obj.setSolid(human.solid)    # Set to wireframe if human is in wireframe
+            # TODO perhaps other properties should be copied from human to object, such as subdivision state. For other hints, and duplicate code, see guicommon Object.setProxy()
 
         # TODO why return both obj and mesh if you can access the mesh easily through obj.mesh?
         return mesh,obj
@@ -192,7 +194,7 @@ class Proxy:
         Reconstruct reverse vertex (and weights) mapping
         """
         self.vertWeights = {}
-        for pxy_vIdx in xrange(self.ref_vIdxs.shape[0]):
+        for pxy_vIdx in range(self.ref_vIdxs.shape[0]):
             _addProxyVertWeight(self.vertWeights, self.ref_vIdxs[pxy_vIdx, 0], pxy_vIdx, self.weights[pxy_vIdx, 0])
             _addProxyVertWeight(self.vertWeights, self.ref_vIdxs[pxy_vIdx, 1], pxy_vIdx, self.weights[pxy_vIdx, 1])
             _addProxyVertWeight(self.vertWeights, self.ref_vIdxs[pxy_vIdx, 2], pxy_vIdx, self.weights[pxy_vIdx, 2])
@@ -242,7 +244,7 @@ class Proxy:
         return self.vertexBoneWeights is not None
 
 
-    def getVertexWeights(self, humanWeights, skel=None):
+    def getVertexWeights(self, humanWeights, skel=None, allowCache=False):
         """
         Map armature weights mapped to the human to the proxy mesh through the
         proxy mapping.
@@ -263,6 +265,10 @@ class Proxy:
         # the bones of the reference skeleton, to those of the current skeleton.
         # The current skeleton is retrieved from the human object linked to this
         # proxy.
+        
+        import time
+        import log
+
         if self.hasCustomVertexWeights():
             # TODO we could introduce caching of weights here as long as the skeleton is not changed
             if skel is None:
@@ -272,25 +278,57 @@ class Proxy:
 
         # Remap weights through proxy mapping
         WEIGHT_THRESHOLD = 1e-4  # Threshold for including bone weight
+
+        recalculate = True
         weights = OrderedDict()
 
-        for bname, (indxs, wghts) in humanWeights.data.items():
-            vgroup = []
-            empty = True
-            for (v,wt) in zip(indxs, wghts):
-                try:
-                    vlist = self.vertWeights[v]
-                except KeyError:
-                    vlist = []
-                for (pv, w) in vlist:
-                    pw = w*wt
-                    if (pw > WEIGHT_THRESHOLD):
-                        vgroup.append((pv, pw))
-                        empty = False
-            if not empty:
-                weights[bname] = vgroup
+        if not allowCache:
+            pass
+            #print("Caching not allowed")
+        else:
+            if self.weightsCache is None:
+                pass
+                #print("There is no cache")
+            else:
+                if not skel is None:
+                    if skel == self.cacheSkel:
+                        recalculate = False
+                    else:
+                        log.debug("The skeleton is different")
+
+        if recalculate:
+            log.debug("remapping weights for proxy " + self.name)
+            start = time.perf_counter()
+            for bname, (indxs, wghts) in list(humanWeights.data.items()):
+                vgroup = []
+                empty = True
+                for (v,wt) in zip(indxs, wghts):
+                    try:
+                        vlist = self.vertWeights[v]
+                    except KeyError:
+                        vlist = []
+                    for (pv, w) in vlist:
+                        pw = w*wt
+                        if (pw > WEIGHT_THRESHOLD):
+                            vgroup.append((pv, pw))
+                            empty = False
+                if not empty:
+                    weights[bname] = vgroup
+            stop = time.perf_counter()
+
+            hw = humanWeights.create(weights)
+            if allowCache:
+                self.weightsCache = hw
+                self.cacheSkel = skel
+            else:
+                self.weightsCache = None
+                self.cacheSkel = None
+
+            log.debug("remapping weights for %s took %.5f seconds", self.name, stop - start)
+        else:
+            hw = self.weightsCache
         
-        return humanWeights.create(weights)#, vertexCount)
+        return hw
 
 
 doRefVerts = 1
@@ -318,7 +356,7 @@ def loadProxy(human, path, type="Clothes"):
                 try:
                     log.message('Compiling binary proxy file %s', npzpath)
                     saveBinaryProxy(proxy, npzpath)
-                except StandardError:
+                except Exception:
                     log.notice('unable to save compiled proxy: %s', npzpath, exc_info=True)
                     if os.path.isfile(npzpath):
                         # Remove file again, in case an empty file is left
@@ -335,15 +373,15 @@ def loadProxy(human, path, type="Clothes"):
     return proxy
 
 def loadTextProxy(human, filepath, type="Clothes"):
-    from codecs import open
     try:
-        fp = open(filepath, "rU", encoding="utf-8")
+        fp = open(filepath, "r", encoding="utf-8")
     except IOError:
         log.error("*** Cannot open %s", filepath)
         return None
 
     folder = os.path.realpath(os.path.expanduser(os.path.dirname(filepath)))
     proxy = Proxy(filepath, type, human)
+    proxy.max_pole = 8
     refVerts = []
 
     status = 0
@@ -387,7 +425,7 @@ def loadTextProxy(human, filepath, type="Clothes"):
             status = doRefVerts
         elif key == 'weights':
             status = doWeights
-            if proxy.weights == None:
+            if proxy.weights is None:
                 proxy.weights = {}
             weights = []
             proxy.weights[words[1]] = weights
@@ -491,11 +529,14 @@ def loadTextProxy(human, filepath, type="Clothes"):
 
         else:
             log.warning('Unknown keyword %s found in proxy file %s', key, filepath)
+    fp.close()
 
     if proxy.z_depth == -1:
         log.warning('Proxy file %s does not specify a Z depth. Using 50.', filepath)
         proxy.z_depth = 50
 
+    # since max-pole is used for the calculation of neighboring planes we have to double it initially
+    proxy.max_pole *= 2
     proxy._finalize(refVerts)
 
     return proxy
@@ -505,71 +546,70 @@ def saveBinaryProxy(proxy, path):
     def _properPath(path):
         return getpath.getJailedPath(path, folder)
 
-    fp = open(path, 'wb')
-    tagStr, tagIdx = _packStringList(proxy.tags)
-    uvStr,uvIdx = _packStringList([ _properPath(proxy.uvLayers[k]) for k in sorted(proxy.uvLayers.keys()) ])
+    with open(path, 'wb') as fp:
+        tagStr, tagIdx = _packStringList(proxy.tags)
+        uvStr,uvIdx = _packStringList([ _properPath(proxy.uvLayers[k]) for k in sorted(proxy.uvLayers.keys()) ])
 
-    licStr, licIdx = proxy.license.toNumpyString()
+        licStr, licIdx = proxy.license.toNumpyString()
 
-    folder = os.path.dirname(path)
+        folder = os.path.dirname(path)
 
 
-    vars_ = dict(
-        #proxyType = np.fromstring(proxy.type, dtype='S1'),     # TODO store proxy type?
-        name = np.fromstring(proxy.name, dtype='S1'),
-        uuid = np.fromstring(proxy.uuid, dtype='S1'),
-        description = np.fromstring(proxy.description, dtype='S1'),
-        basemesh = np.fromstring(proxy.basemesh, dtype='S1'),
-        tags_str = tagStr,
-        tags_idx = tagIdx,
-        lic_str = licStr,
-        lic_idx = licIdx,
-        uvLayers_str = uvStr,
-        uvLayers_idx = uvIdx,
-        obj_file = np.fromstring(_properPath(proxy.obj_file), dtype='S1'),
-        version = np.asarray(proxy.version, dtype=np.int32)
-    )
+        vars_ = dict(
+            #proxyType = np.fromstring(proxy.type, dtype='S1'),     # TODO store proxy type?
+            name = np.fromstring(proxy.name, dtype='S1'),
+            uuid = np.fromstring(proxy.uuid, dtype='S1'),
+            description = np.fromstring(proxy.description, dtype='S1'),
+            basemesh = np.fromstring(proxy.basemesh, dtype='S1'),
+            tags_str = tagStr,
+            tags_idx = tagIdx,
+            lic_str = licStr,
+            lic_idx = licIdx,
+            uvLayers_str = uvStr,
+            uvLayers_idx = uvIdx,
+            obj_file = np.fromstring(_properPath(proxy.obj_file), dtype='S1'),
+            version = np.asarray(proxy.version, dtype=np.int32)
+        )
 
-    if proxy.material_file:
-        vars_["material_file"] = np.fromstring(_properPath(proxy.material_file), dtype='S1')
+        if proxy.material_file:
+            vars_["material_file"] = np.fromstring(_properPath(proxy.material_file), dtype='S1')
 
-    if np.any(proxy.deleteVerts):
-        vars_["deleteVerts"] = proxy.deleteVerts
+        if np.any(proxy.deleteVerts):
+            vars_["deleteVerts"] = proxy.deleteVerts
 
-    if proxy.z_depth is not None and proxy.z_depth != -1:
-        vars_["z_depth"] = np.asarray(proxy.z_depth, dtype=np.int32)
+        if proxy.z_depth is not None and proxy.z_depth != -1:
+            vars_["z_depth"] = np.asarray(proxy.z_depth, dtype=np.int32)
 
-    if proxy.max_pole:
-        vars_["max_pole"] = np.asarray(proxy.max_pole, dtype=np.uint32)
+        if proxy.max_pole:
+            vars_["max_pole"] = np.asarray(proxy.max_pole, dtype=np.uint32)
 
-    proxy.tmatrix.toNumpyStruct(vars_)
+        proxy.tmatrix.toNumpyStruct(vars_)
 
-    special_poses = []
-    for posetype, posename in proxy.special_pose.items():
-        special_poses.append(posetype)
-        special_poses.append(posename)
-    specialposeStr, specialposeIdx = _packStringList(special_poses)
-    vars_["special_pose_str"] = specialposeStr
-    vars_["special_pose_idx"] = specialposeIdx
+        special_poses = []
+        for posetype, posename in list(proxy.special_pose.items()):
+            special_poses.append(posetype)
+            special_poses.append(posename)
+        specialposeStr, specialposeIdx = _packStringList(special_poses)
+        vars_["special_pose_str"] = specialposeStr
+        vars_["special_pose_idx"] = specialposeIdx
 
-    if proxy.weights[:,1:].any():
-        # 3 ref verts used in this proxy
-        num_refverts = 3
-        vars_["ref_vIdxs"] = proxy.ref_vIdxs
-        vars_["offsets"] = proxy.offsets
-        vars_["weights"] = proxy.weights
-    else:
-        # Proxy uses exact fitting exclusively: store npz file more compactly
-        num_refverts = 1
-        vars_["ref_vIdxs"] = proxy.ref_vIdxs[:,0]
-        vars_["weights"] = proxy.weights[:,0]
-    vars_['num_refverts'] = np.asarray(num_refverts, dtype=np.int32)
+        if proxy.weights[:,1:].any():
+            # 3 ref verts used in this proxy
+            num_refverts = 3
+            vars_["ref_vIdxs"] = proxy.ref_vIdxs
+            vars_["offsets"] = proxy.offsets
+            vars_["weights"] = proxy.weights
+        else:
+            # Proxy uses exact fitting exclusively: store npz file more compactly
+            num_refverts = 1
+            vars_["ref_vIdxs"] = proxy.ref_vIdxs[:,0]
+            vars_["weights"] = proxy.weights[:,0]
+        vars_['num_refverts'] = np.asarray(num_refverts, dtype=np.int32)
 
-    if proxy.vertexBoneWeights_file:
-        vars_['vertexBoneWeights_file'] = np.fromstring(_properPath(proxy.vertexBoneWeights_file), dtype='S1')
+        if proxy.vertexBoneWeights_file:
+            vars_['vertexBoneWeights_file'] = np.fromstring(_properPath(proxy.vertexBoneWeights_file), dtype='S1')
 
-    np.savez_compressed(fp, **vars_)
-    fp.close()
+        np.savez_compressed(fp, **vars_)
     os.utime(path, None)  # Ensure modification time is updated
 
 def loadBinaryProxy(path, human, type):
@@ -583,12 +623,12 @@ def loadBinaryProxy(path, human, type):
 
     proxy = Proxy(path, proxyType, human)
 
-    proxy.name = npzfile['name'].tostring()
-    proxy.uuid = npzfile['uuid'].tostring()
-    proxy.basemesh = npzfile['basemesh'].tostring()
+    proxy.name = str(npzfile['name'].tostring(), 'utf8')
+    proxy.uuid = str(npzfile['uuid'].tostring(), 'utf8')
+    proxy.basemesh = str(npzfile['basemesh'].tostring(), 'utf8')
 
     if 'description' in npzfile:
-        proxy.description = npzfile['description'].tostring()
+        proxy.description = str(npzfile['description'].tostring(), 'utf8')
 
     if 'version' in npzfile:
         proxy.version = int(npzfile['version'])
@@ -637,14 +677,14 @@ def loadBinaryProxy(path, human, type):
 
     proxy.material = material.Material(proxy.name)
     if 'material_file' in npzfile:
-        proxy._material_file = npzfile['material_file'].tostring()
+        proxy._material_file = str(npzfile['material_file'].tostring(), 'utf8')
     if proxy.material_file:
         proxy.material.fromFile(proxy.material_file)
 
-    proxy._obj_file = npzfile['obj_file'].tostring()
+    proxy._obj_file = str(npzfile['obj_file'].tostring(), 'utf8')
 
     if 'vertexBoneWeights_file' in npzfile:
-        proxy._vertexBoneWeights_file = npzfile['vertexBoneWeights_file'].tostring()
+        proxy._vertexBoneWeights_file = str(npzfile['vertexBoneWeights_file'].tostring(), 'utf8')
         if proxy.vertexBoneWeights_file:
             from animation import VertexBoneWeights
             proxy.vertexBoneWeights = VertexBoneWeights.fromFile(proxy.vertexBoneWeights_file)
@@ -787,8 +827,8 @@ class TMatrix:
         """Deserialize TMatrix from npz file"""
         def _unpack_scales(scales, vidxs):
             scaleData = [None, None, None]
-            for i in xrange(3):
-                if i >= min(len(scales), len(vidxs)/2):
+            for i in range(3):
+                if i >= min(len(scales), len(vidxs)//2):
                     break
                 scale = scales[i]
                 if not math.isnan(scale):
@@ -798,8 +838,8 @@ class TMatrix:
 
         def _unpack_shears(shears, vidxs):
             shearData = [None, None, None]
-            for i in xrange(3):
-                if i >= min(len(scales)/2, len(vidxs)/2):
+            for i in range(3):
+                if i >= min(len(shears)//2, len(vidxs)//2):
                     break
                 shear1, shear2 = shears[i*2], shears[i*2+1]
                 vidx1, vidx2 = vidxs[i*2], vidxs[i*2+1]
@@ -924,26 +964,20 @@ def transferVertexMaskToProxy(vertsMask, proxy):
     that have all vertices hidden will be hidden.
     True in vertex mask means: show vertex, false means hide (masked)
     """
-    # Convert basemesh vertex mask to local mask for proxy vertices
-    proxyVertMask = np.ones(len(proxy.ref_vIdxs), dtype=bool)
-
-    # Proxy verts that use exact mapping
+    # preset new mask to unhidden and calculate all the vertices which depend directly to base mesh
+    # (value 1.0, 0.0, 0.0 in weighting), create this mask with negation using np.any
+    masksize = len(proxy.ref_vIdxs)
+    proxyVertMask = np.ones(masksize, dtype=bool)
     exact_mask = ~np.any(proxy.weights[:,1:], axis=1)
 
-    # Faster numpy implementation of the above:
-    unmasked_row_col = np.nonzero(vertsMask[proxy.ref_vIdxs])
-    unmasked_rows = unmasked_row_col[0]
-    if len(unmasked_rows) > 0:
-        unmasked_count = np.bincount(unmasked_rows) # count number of unmasked verts per row
-        # only hide/mask a vertex if at least two referenced body verts are hidden/masked
-        masked_idxs = np.nonzero(unmasked_count < 2)
-        proxyVertMask[masked_idxs] = False
-    else:
-        # All verts are masked
-        proxyVertMask[:] = False
-
-    # Directly map exactly mapped proxy verts
-    proxyVertMask[exact_mask] = vertsMask[proxy.ref_vIdxs[exact_mask,0]]
+    # now set values according to mask
+    for c in range(masksize):
+        p = proxy.ref_vIdxs[c]
+        if exact_mask[c]:
+            proxyVertMask[c] = vertsMask[p[0]]
+        else:
+            if (int(vertsMask[p[0]]) + int(vertsMask[p[1]]) + int(vertsMask[p[2]])) < 2:
+                proxyVertMask[c] = False            # at least two visible Reference
 
     return proxyVertMask
 
@@ -977,7 +1011,7 @@ def peekMetadata(proxyFilePath, proxyType=None):
             # Binary proxy file
             npzfile = np.load(proxyFilePath)
 
-            uuid = npzfile['uuid'].tostring()
+            uuid = str(npzfile['uuid'].tostring(), 'utf8')
             tags = set(_unpackStringList(npzfile['tags_str'], npzfile['tags_idx']))
             return (uuid, tags)
         except Exception as e:
@@ -985,21 +1019,19 @@ def peekMetadata(proxyFilePath, proxyType=None):
             log.warning("Problem loading metadata from binary proxy, trying ASCII file: %s", e, exc_info=showTrace)
 
     # ASCII proxy file
-    from codecs import open
-    fp = open(proxyFilePath, 'rU', encoding="utf-8")
-    uuid = None
-    tags = set()
-    for line in fp:
-        words = line.split()
-        if len(words) == 0:
-            pass
-        elif words[0] == 'uuid':
-            uuid = words[1]
-        elif words[0] == 'tag':
-            tags.add(" ".join(words[1:]).lower())
-        elif words[0] == 'verts':
-            break
-    fp.close()
+    with open(proxyFilePath, 'r', encoding="utf-8") as fp:
+        uuid = None
+        tags = set()
+        for line in fp:
+            words = line.split()
+            if len(words) == 0:
+                pass
+            elif words[0] == 'uuid':
+                uuid = words[1]
+            elif words[0] == 'tag':
+                tags.add(" ".join(words[1:]).lower())
+            elif words[0] == 'verts':
+                break
     return (uuid, tags)
 
 
@@ -1007,7 +1039,8 @@ def _packStringList(strings):
     text = ''
     index = []
     for string in strings:
-        index.append(len(text))
+        asbytes = bytearray(text,'utf-8')
+        index.append(len(asbytes))
         text += string
     text = np.fromstring(text, dtype='S1')
     index = np.array(index, dtype=np.uint32)
@@ -1018,11 +1051,11 @@ def _unpackStringList(text, index):
     last = None
     for i in index:
         if last is not None:
-            name = text[last:i].tostring()
+            name = str(text[last:i].tostring(), 'utf8')
             strings.append(name)
         last = i
     if last is not None:
-        name = text[last:].tostring()
+        name = str(text[last:].tostring(), 'utf8')
         strings.append(name)
 
     return strings
@@ -1042,7 +1075,7 @@ def _getFilePath(filename, folder = None, altExtensions=None):
                 path = os.path.splitext(aPath)[0]+orgExt
                 return getpath.formatPath(path)
 
-    if not filename or not isinstance(filename, basestring):
+    if not filename or not isinstance(filename, str):
         return filename
 
     searchPaths = []
